"""This is just some sample code that was used to explore interacting with
the keycloak api.  Eventually throw away, but is useful to go back to when
trying to figure out best way to integrate.
"""

import keycloak
import src.constants as constants
import keycloak_wrapper
import requests

def test_keycloak_module():
    # works
    keycloak_openid = keycloak.KeycloakOpenID(
                        server_url=constants.KC_HOST,
                        client_id=constants.KC_CLIENTID,
                        realm_name=constants.KC_REALM,
                        client_secret_key=constants.KC_SECRET,
                        verify=True)

    print(f'keycloak_openid: {keycloak_openid}')

def getAccessToken():
    uri = f"{constants.KC_HOST}/auth/realms/{constants.KC_REALM}/protocol/openid-connect/token"
    header = {'Accept': 'application/json'}
    params = {
              "client_id": constants.KC_CLIENTID,
              "client_secret": constants.KC_SECRET,
              "grant_type":"client_credentials"}
    print(f'uri: {uri}')
    r = requests.post(uri, data=params, headers=header)
    print(f'status_code: {r.status_code}')
    #print(r.json())
    access_key = r.json()
    accessToken = access_key['access_token']
    print(f'response as json string {accessToken}')
    return accessToken

def test_keycloak_wrapper_module():
    uri = f"{constants.KC_HOST}/auth/realms/{constants.KC_REALM}/protocol/openid-connect/token"
    header = {'Accept': 'application/json'}
    params = {
              "client_id": constants.KC_CLIENTID,
              "client_secret": constants.KC_SECRET,
              "grant_type":"client_credentials"}
    print(f'uri: {uri}')
    r = requests.post(uri, data=params, headers=header)
    print(f'status_code: {r.status_code}')
    #print(r.json())
    access_key = r.json()
    accessToken = access_key['access_token']
    print(f'response as json string {accessToken}')

    kc_url = f"{constants.KC_HOST}/auth/"
    #create_role(KEYCLOAK_URL, REALM_NAME, ADMIN_REALM_TOKEN, CLIENT_NAME, NEW_ROLE_NAME)
    #keycloak_wrapper.create_role(kc_url, constants.KC_REALM, accessToken, constants.KC_CLIENTID, "testrole")

    #keycloak_wrapper.access_token(KEYCLOAK_URL, REALM_NAME, CLIENT_NAME,CLIENT_SECRET, USERNAME, PASSWORD)
    ADMIN_CLIENTS = f"auth/admin/realms/{constants.KC_REALM}/clients"
    url = f'{constants.KC_HOST}/{ADMIN_CLIENTS}'
    print(f"\nurl: {url}")
    params = {"realm-name": constants.KC_REALM}
    headers = {"Authorization": "Bearer " + accessToken}
    r = requests.get(url=url, params=params, headers=headers)
    print(f"status code: {r.status_code}")
    print(f'respons: {r}')
    #print(f"json: {r.json()}")

    keycloak_wrapper.create_role(kc_url, constants.KC_REALM, accessToken, constants.KC_CLIENTID, "testrole", 'test role description')

def add_many_roles():
    accessToken = getAccessToken()
    kc_url = f"{constants.KC_HOST}/auth/"

    roleFile = '/home/kjnether/proj/keycloak-add-role/demo_roles.txt'
    with open(roleFile, 'r') as fh:
        for inLine in fh:
            inLine = inLine.replace("\n", '').replace('(', '').replace(')','').replace("CURRENT_USER", '').strip()
            inLine = inLine.replace("'", '')
            inLine = inLine[:-2]
            inList = inLine.split(',')
            orgNumber = inList[0]
            orgName = inList[1]
            roleName = f'fom_forest_client_{str(orgNumber)}'
            keycloak_wrapper.create_role(kc_url, constants.KC_REALM, accessToken, constants.KC_CLIENTID, roleName, orgName)


if __name__ == '__main__':
    #test_keycloak_wrapper_module()
    add_many_roles()


import logging

import pytest

from . import ForestClient

LOGGER = logging.getLogger(__name__)
LOGGER.debug("message test")


class Test_ForestClientFromGit:

    def test_forestClientFromGitParse(self, forestClient_fixture):
        forestClient_fixture.parseMultiple()
        LOGGER.debug(
            "number of forest clients: " +
            f"{len(forestClient_fixture.forestClientData)}")
        assert len(forestClient_fixture.forestClientData) >= 22408

    def test_getMatchingClient(self, forestClientParsed_fixture):
        values = forestClientParsed_fixture.getMatchingClient('arm')
        LOGGER.debug(f"values: {values}")

    def test_getPaddedForestClientID(self, forestClientUtil_fixture):
        fc = forestClientUtil_fixture
        sampleData = [['33', '00000033'], ['2234', '00002234']]
        for data in sampleData:
            padded = fc.getPaddedForestClientID(data[0])
            assert padded == data[1]

    def test_getForestClientDescription(self, forestClientParsed_fixture):
        client = 187347
        desc = forestClientParsed_fixture.getForestClientDescription(client)
        LOGGER.debug(f"desc: {desc}")
        assert desc is not None
        assert desc == 'LANDING HV DEVELOPMENT LTD'

    def test_forestClientIdExists(self, forestClientParsed_fixture):
        fc = forestClientParsed_fixture
        clientsThatExist = ['188736', '187347', '186040', '181461', '177380',
                            '62911', '23530', '2884', '1011']
        clientsThatDontExist = ['88348', '88978', '90125']
        for clExist in clientsThatExist:
            LOGGER.debug(f"fcExists: {clExist}")

            assert fc.forestClientIdExists(clExist) is True

        for clExist in clientsThatDontExist:
            fcExists = fc.forestClientIdExists(clExist)
            LOGGER.debug(f"fcExists: {clExist}  :  {fcExists}")
            assert fcExists is False


@pytest.fixture
def forestClient_fixture():
    return ForestClient.ForestClientFromGit()


@pytest.fixture
def forestClientParsed_fixture(forestClient_fixture):
    forestClient_fixture.parseMultiple()
    return forestClient_fixture


@pytest.fixture
def forestClientUtil_fixture():
    fcUtil = ForestClient.ForestClientUtil()
    return fcUtil


@pytest.fixture
def forestClientCsv_fixture():
    fcUtil = ForestClient.ForestClientFromCsv()
    return fcUtil



import os
import dotenv
import sys
import logging

LOGGER = logging.getLogger(__name__)

envFile = '.env-dev'

# populate the env vars from an .env file if it exists
envPath = os.path.join(os.path.dirname(__file__), '..', '..', envFile)
localEnvPath = os.path.join(os.getcwd(), '.env')
LOGGER.debug(f"envPath: {envPath}")
if os.path.exists(envPath):
    LOGGER.debug(f"loading dot env {envPath}...")
    dotenv.load_dotenv(envPath)

elif os.path.exists(localEnvPath):
    LOGGER.debug(f"loading dot env {localEnvPath}...")
    dotenv.load_dotenv(localEnvPath)

# env vars that should be populated for script to run
ENV_VARS = ['KC_HOST', 'KC_CLIENTID', 'KC_REALM', 'KC_SECRET',
            'KC_FOM_CLIENTID']
# KC_SA_CLIENTID

module = sys.modules[__name__]

envsNotSet = []
for env in ENV_VARS:
    if env not in os.environ:
        envsNotSet.append(env)
    else:
        # transfer env vars to module properties
        setattr(module, env, os.environ[env])

if envsNotSet:
    msg = 'The script expects the following environment variables to ' + \
          f'be set {envsNotSet}'
    raise EnvironmentError(msg)

# default values for the references to the forest clients in the fom repo
FOREST_CLIENT_IN_GIT = \
    'https://raw.githubusercontent.com/bcgov/nr-fom-api/master/apps/api/src/migrations/main/1616015261635-forestClient.js' + '||' + \
    'https://github.com/bcgov/nr-fom-api/blob/master/apps/api/src/migrations/main/1639180924469-forestClientTypesNonCNonI.js' # noqa

FOM_CLIENT_ID = 'fom'



'''
as a proof of concept that roles could be created using a script, created a
bunch of role in the client that was used to gain access to keycloak admin

This script cleans up those roles
'''

import logging

from . import constants
from . import FOMKeyCloak

LOGGER = logging.getLogger(__name__)


class CleanupRoles:

    def __init__(self):
        self.kc = FOMKeyCloak.FomKeycloak()

    def getRoles(self):
        roles = self.kc.getRoles(constants.KC_SA_CLIENTID)
        roleNames = [role['name'] for role in roles]
        roleNames.sort()

        LOGGER.debug(f"Num roles: {len(roleNames)}")
        return roleNames

    def removeRoles(self, roleNames):
        client = self.kc.getClient(constants.KC_SA_CLIENTID)
        for roleName in roleNames:
            self.kc.removeRole(client['id'], roleName)


if __name__ == '__main__':
    LOGGER = logging.getLogger()
    LOGGER.setLevel(logging.DEBUG)
    hndlr = logging.StreamHandler()
    formatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(lineno)d - %(message)s')
    hndlr.setFormatter(formatter)
    LOGGER.addHandler(hndlr)

    LOGGER.debug("hello")
    cr = CleanupRoles()
    roles = cr.getRoles()
    cr.removeRoles(roles)



""" interface to provision of forest client data.  For now its going to
parse / the data in the fom github repo...

Eventually will interface with the forest client api to get this information
"""

import logging
import operator
import re

import requests
try:
    from . import constants
except ImportError:
    import constants

LOGGER = logging.getLogger(__name__)


class ForestClientFromGit:

    def __init__(self):
        self.fcTable = constants.FOREST_CLIENT_IN_GIT
        self.forestClientData = {}
        self.fcUtil = ForestClientUtil()
        self.parseMultiple()

    def parseMultiple(self):
        files2Parse = constants.FOREST_CLIENT_IN_GIT.split("||")
        LOGGER.debug(f"files2Parse:{files2Parse}")
        for file2Parse in files2Parse:
            self.parse(file2Parse)

    def parse(self, file2Parse):
        """pulls down the js migration file, parses out the forest clients from
        it and adds them to the 'forestClientData' property

        :param file2Parse: url to the migrations js file that has the forest
                           clients in it
        :type file2Parse: str
        """
        # pulling the data down from the git repo
        LOGGER.debug(f"file2Parse: {file2Parse}")
        response = requests.get(file2Parse)
        response.raise_for_status()
        # response.raise_for_status()
        LOGGER.debug(f"status_code: {response.status_code}")
        jsFCFile = response.text
        # the insert line marks the start of the data.  This regex detects
        # the insert line
        # sample line that shows the pattern that the next line does.
        #  ('189974', 'LUXOR-SPUR DEVELOPMENTS LTD.', CURRENT_USER),
        dataLine_regex = re.compile(
            "^\s+\('[0-9]{4,8}'\,\s+'.+'\,\s+CURRENT_USER\)\,\s*$")  # noqa
        LOGGER.debug(f"jsFCFile {type(jsFCFile)}")
        for line in jsFCFile.split('\n'):
            if dataLine_regex.match(line):
                line = line.replace(', CURRENT_USER),', '')
                line = line.replace('(', '').replace(')', '').strip()
                line = line.replace(', ', ',').replace("'", '')
                lineList = line.split(',')
                forestClientId = self.fcUtil.getPaddedForestClientID(
                    lineList[0])
                self.forestClientData[lineList[1]] = forestClientId
        LOGGER.debug(f"number forest clients = {len(self.forestClientData)}")

    def getForestClientDescription(self, clientId):
        clientId = self.fcUtil.getPaddedForestClientID(clientId)
        returnVal = None
        for fc in self.forestClientData:
            if clientId == self.forestClientData[fc]:
                returnVal = fc
                break
        return returnVal

    def getMatchingClient(self, characters):
        values = []
        for fc in self.forestClientData:
            if characters.lower() in fc.lower():
                values.append([fc, self.forestClientData[fc]])
        values = (sorted(values, key=operator.itemgetter(0)))
        return values

    def forestClientIdExists(self, clientId):
        # remove any padding characters
        clientIdPadded = self.fcUtil.getPaddedForestClientID(clientId)
        clientExists = False
        if clientIdPadded in self.forestClientData.values():
            clientExists = True
        return clientExists


class ForestClientUtil:
    def __init__(self):
        self.rolePrefix = 'fom_forest_client_'

    def getPaddedForestClientID(self, clientId):
        """Forest clients are an 8 digit field that is stored as a string.
        This method will pad the forestclient id with leading 0's to meet
        the expected 8 character length

        :param clientId: input forest client
        :type clientId: str, int
        """
        clientId_str = str(clientId)
        numChars = len(clientId_str)

        if numChars < 8:
            padding = 8 - numChars
            clientId_str = ('0' * padding) + clientId_str
        return clientId_str

    def getRoleName(self, clientID):
        fcPadded = self.getPaddedForestClientID(clientID)
        roleName = f'{self.rolePrefix}{fcPadded}'
        return roleName


class ForestClient(ForestClientFromGit):
    # TODO: should defined an abstract class that standardizes the function
    #       calls so can pivot to new data source for forest client when
    #       it becomes available

    def __init__(self):
        ForestClientFromGit.__init__(self)

    # def getMatchingClient(self, searchCharacters):
    #     return self.fc_git.getMatchingClient(searchCharacters)

    # def forestClientIdExists(self, clientId):
    #     return self.fc_git.forestClientIdExists(clientId)

    # def getForestClientDescription(self, clientId):
    #     """Returns the description for a matching forest client id"""
    #     return self.fc_git.getForestClientDescription(clientId)



import logging

import keycloak_wrapper
# TODO: not happy with the way this wrapper is implemented... Should provide
#       errors when api call fails, among other things.  Connecting to api
#       end points in KC is actually pretty easy.  Move to wrapping my self.
import requests

try:
    from . import constants
    from . import ForestClient
except ImportError:
    import constants
    import ForestClient

LOGGER = logging.getLogger(__name__)


class FomKeycloak:

    def __init__(self):
        #self.connect()
        self.getAccessToken()
        self.fcUtil = ForestClient.ForestClientUtil()

    # def connect(self):
    #     self.token = keycloak_wrapper.access_token_sa(
    #         constants.KC_HOST,
    #         constants.KC_REALM,
    #         constants.KC_CLIENTID,
    #         constants.KC_SECRET)
    #     self.access_token = self.token['access_token']
    #     LOGGER.debug("success getting access token")

    def getAccessToken(self):
        uri = f"{constants.KC_HOST}/auth/realms/{constants.KC_REALM}/protocol/openid-connect/token"
        header = {'Accept': 'application/json'}
        params = {
                "client_id": constants.KC_CLIENTID,
                "client_secret": constants.KC_SECRET,
                "grant_type":"client_credentials"}
        LOGGER.debug(f'uri: {uri}')
        r = requests.post(uri, data=params, headers=header)
        r.raise_for_status()
        access_key = r.json()
        self.access_token = access_key['access_token']
        LOGGER.debug(f'response as json string {self.access_token}')

    def getMatchingUsers(self, userId):
        """ Keycloak contains a lot of information about users.  This method
        determines if a userid exists in keycloak.  The method will do its own
        search of all the users in keycloak.  (not efficient)

        Looks for either <userid>@<identity provider>, or looks for any user id
        that matches the identity provider.

        If more than one user is found then a warning message will be logged.

            user@dir
            user@bceid
            <email address>

        Will get a list of the users in the realm and search for

        :param userId: [description]
        :type userId: [type]
        :return: [description]
        :rtype: [type]
        """
        # /{realm}/users
        # params = {"realm-name": constants.KC_REALM}
        # headers = {"Authorization": "Bearer " + self.access_token}
        # Url = f"{constants.KC_HOST}/auth/admin/realms/{constants.KC_REALM}/users" # noqa
        # LOGGER.debug(f"URL: {Url}")
        # response = requests.get(url=Url,
        #                         headers=headers)

        # LOGGER.debug(f"response: {response}")
        # LOGGER.debug(f"status: {response.status_code}")
        # LOGGER.debug(f"status: {response.json()}")

        users = self.getAllUsers()

        # if userid.lower() in [ role['name'] for user in users]:

        # search in username email attributes.idir_username
        # return username / email
        matchedUsers = []
        for user in users:

            if user['username'].lower().startswith(userId.lower()):
                email = ''
                if email in user:
                    email = user['email']
                matchedUsers.append([user['username'], email])
            elif ('email' in user) and user['email'].lower().startswith(
                    userId.lower()):
                matchedUsers.append([user['username'], user['email']])
            elif (('attributes' in user) and
                  'idir_username' in user['attributes']) and \
                    user['attributes']['idir_username'][0].lower().startswith(
                        userId.lower()):
                matchedUsers.append([user['username'], user['email']])

        LOGGER.debug(f"users: {users}")
        return matchedUsers

    def getAllUsers(self):
        users = keycloak_wrapper.realm_users(
            f"{constants.KC_HOST}/auth/",
            constants.KC_REALM,
            self.access_token)
        return users

    def isValidUser(self, userid):
        """validates that the user provided exists in keycloak, and that the
        id is unique

        :param userid: input user id to be validated
        :type userid: str
        """
        isValid = False
        users = self.getAllUsers()
        LOGGER.debug(f"users: {users}")
        matches = []
        for user in users:
            if user['username'] == userid:
                matches.append(user)
        if len(matches) == 1:
            isValid = True
        return isValid

    def getRoles(self, clientID):
        """returns a list of roles that exist within the provided client id"""
        roles = keycloak_wrapper.client_roles(
            f"{constants.KC_HOST}/auth/",
            constants.KC_REALM,
            self.access_token,
            clientID)
        return roles

    def getFOMRoles(self, forestClientNumber: int):
        """_summary_

        :param forestClientNumber: _description_
        :type forestClientNumber: int
        :param clientID: _description_, defaults to 'fom'
        :type clientID: str, optional
        :return: _description_
        :rtype: _type_
        """
        roles = self.getRoles(constants.FOM_CLIENT_ID)
        roleName = self.fcUtil.getRoleName(forestClientNumber)

        filteredRoles = [role for role in roles if roleName == role['name']]
        return filteredRoles

    def roleExists(self, forestClientNumber: int):
        """ Checks to see if the forest client exists in keycloak

        :param forestClientNumber: [description]
        :type forestClientNumber: int
        """
        roleName = self.fcUtil.getRoleName(forestClientNumber)

        roles = self.getFOMRoles(forestClientNumber)
        roleExists = False
        if roleName in [role['name'] for role in roles]:
            roleExists = True
        return roleExists

    def getFomClientId(self):
        """Looks up the FOM client 'id' using the 'clientid'

        the client 'id' is usually what is required to create / modify objects
        in / for / on behalf of the client
        """
        clients = keycloak_wrapper.realm_clients(
            f"{constants.KC_HOST}/auth/", constants.KC_REALM,
            self.access_token)
        fomClient = None
        for client in clients:
            if client['clientId'] == constants.KC_FOM_CLIENTID:
                fomClient = client['id']
        return fomClient

    def createRole(self, forestClientId, description):
        """Creates the role for the forest client id if it doesn't already
        exist.

        * send payload/body where {"name":"role name to create"}
        * end point /auth/admin/realms/$REALM/clients/$CLIENTID/roles
        * method POST
        """
        # self.fcUtil.
        if not self.roleExists(forestClientId):
            roleName = self.fcUtil.getRoleName(forestClientId)
            LOGGER.debug(f'rolename: {roleName}')

            clientid = self.getFomClientId()
            Url = f"{constants.KC_HOST}/auth/admin/realms/{constants.KC_REALM}/clients/{clientid}/roles"  # noqa
            data = {"name": roleName,
                    "description":  description}
            headers = {
                "Authorization": "Bearer " + self.access_token,
                'Content-type': 'application/json',
                'Accept': 'application/json'}
            response = requests.post(url=Url,
                                     headers=headers,
                                     json=data)
            response.raise_for_status()

    def removeRole(self, clientID, roleName):
        """_summary_

        :param clientID: _description_
        :type clientID: _type_
        :param roleName: _description_
        :type roleName: _type_

        https://$KC_URL/auth/admin/realms/$REALM/clients/{id}/roles/{role-name}

        """
        LOGGER.debug(f'clientID: {clientID}')
        LOGGER.debug(f'roleName: {roleName}')
        Url = f"{constants.KC_HOST}/auth/admin/realms/{constants.KC_REALM}/clients/{clientID}/roles/{roleName}"  # noqa
        headers = {
            "Authorization": "Bearer " + self.access_token,
            'Content-type': 'application/json',
            'Accept': 'application/json'}
        LOGGER.debug(f"deleting the role: {roleName}")
        response = requests.delete(url=Url,
                                   headers=headers
                                   )
        response.raise_for_status()
        LOGGER.debug(f"response: {response.status_code}")

    def getClients(self):
        '''
        GET /{realm}/clients
        '''
        Url = f"{constants.KC_HOST}/auth/admin/realms/{constants.KC_REALM}/clients"  # noqa
        headers = {
            "Authorization": "Bearer " + self.access_token,
            'Content-type': 'application/json',
            'Accept': 'application/json'}
        response = requests.get(url=Url,
                                headers=headers
                                )
        response.raise_for_status()
        LOGGER.debug(f"response: {response.status_code}")

        data = response.json()
        return data

    def getClient(self, clientID):
        """gets a list of all the clients in the realm and returns only the
        client that matches the clientID provided
        """
        clients = self.getClients()
        client = None
        for client in clients:
            if client['clientId'].lower() == clientID.lower():
                break
        return client

    def addRoleToUser(self, userid, forestClientId):
        """This is the role mapping exercise...

        /auth/admin/realms/$REALM/users/$USERID/role-mappings/clients/$CLIENTID
        USERID - comes from user['id']

        assumes that the forestclientid and the userid have been
        validated then does the role mapping

        https://$KC_URL/auth/admin/realms/$REALM/users/$userid/role-mappings/clients/$fom_client_id

        1. get user id
        1. get client id
        1. get role
        """
        users = self.getAllUsers()
        matchUsers = []
        for user in users:
            if user['username'] == userid:
                matchUsers.append(user)

        # TODO: make sure only one user
        LOGGER.debug(f"users length {len(matchUsers)}")

        roles = self.getFOMRoles(forestClientId)
        LOGGER.debug(f"roles length {len(roles)}")
        LOGGER.debug(f"role length {roles}")

        clientid = self.getFomClientId()

        Url = f"{constants.KC_HOST}/auth/admin/realms/{constants.KC_REALM}/users/{matchUsers[0]['id']}/role-mappings/clients/{clientid}"  # noqa
        headers = {
            "Authorization": "Bearer " + self.access_token,
            'Content-type': 'application/json',
            'Accept': 'application/json'}

        response = requests.post(url=Url,
                                 headers=headers,
                                 json=roles)
        response.raise_for_status()



"""command line based tool that attempts to make it easy to add new users
to the fom application
"""

import argparse
import logging
import sys

try:
    from . import FOMKeyCloak
    from . import ForestClient
except ImportError:
    import FOMKeyCloak
    import ForestClient

LOGGER = logging.getLogger()


class CLI:

    def __init__(self):
        pass

    def defineParser(self):
        examples = """Query for forest clients:
                      %(prog)s -qfc acmeforest

                      Query for users:
                      %(prog)s -qu bill

                      Add User:
                      %(prog)s -a 1011 -u bill.the.cat"""

        parser = argparse.ArgumentParser(
            description='Add / Query Fom user data.',
            epilog=examples,
            formatter_class=argparse.RawDescriptionHelpFormatter)
        parser.add_argument(
            '-qfc', '--query-forest-client', type=str,
            help='Define the starting characters for forest clients you ' +
                 'want to view / retrieve forest client ids for')
        parser.add_argument(
            '-qu', '--query-users',
            help='Query for keycloak users that match the string')
        parser.add_argument(
            '-a', '--add-user',
            metavar=('user-to-add', 'forest-client-id'),
            nargs=2,
            help='user is the username in k/c,  forest client id is just' +
                 ' the number')

        # TODO: Add a subparser here to better describe the two args for
        # add-user

        args = parser.parse_args()

        if not args.query_forest_client and \
            not args.query_users and \
                not args.add_user:
            parser.print_help()
            sys.exit()

        LOGGER.debug(f'parser: {parser}')

        LOGGER.debug(f'args: {args}')

        if args.query_forest_client:
            # do search
            LOGGER.debug(f'search chars: {args.query_forest_client}')
            self.queryForestClient(args.query_forest_client)

        elif args.query_users:
            LOGGER.debug(f'search chars: {args.query_users}')
            self.queryUsers(args.query_users)

        else:
            # Adding user, user = 0 fc = 1
            LOGGER.debug(f"adduser arg: {args.add_user}")
            LOGGER.debug(f"adding the user: {args.add_user[0]} to the role " +
                         f"mapping for {args.add_user[1]}")
            self.addUser(args.add_user[0], args.add_user[1])

    def queryForestClient(self, queryString):
        fc = ForestClient.ForestClient()
        matches = fc.getMatchingClient(queryString)
        print(f"forest clients matching: {queryString}")
        print("-"*80)
        formattedList = [
            f"{match[0]:50} - {int(match[1]):8d}" for match in matches]
        print('\n'.join(formattedList))

    def queryUsers(self, queryString):
        kc = FOMKeyCloak.FomKeycloak()
        users = kc.getMatchingUsers(queryString)
        formattedList = [f"{match[0]:35} - {match[1]:20}" for match in users]
        print(f"matching users for search: {queryString}")
        print("-"*80)
        print('\n'.join(formattedList))

    def addUser(self, userid, forestclient):
        """receives a key cloak user id, verifies that it exists and that it
        is unique.

        Does a search to make sure the forest client id exists.

        If both of the above criteria are met, checks to see if a role
        associated with the user already exists.  If not one is created.  Then
        adds the user to the role.

        :param userid: name of input user
        :type userid: str
        :param forestclient: name of forest client
        :type forestclient: str, int
        """
        # validation: verify forest client
        fc = ForestClient.ForestClient()
        if not fc.forestClientIdExists(forestclient):
            msg = f"The forest client: {forestclient} does not exist"
            raise ValueError(msg)

        # validation: verify the user
        kc = FOMKeyCloak.FomKeycloak()
        if not kc.isValidUser(userid):
            msg = f'The key cloak user: {userid} is invalid'
            raise ValueError(msg)

        # adding the user
        if not kc.roleExists(forestclient):
            description = fc.getForestClientDescription(forestclient)
            # creating the role if it doesn't exist
            kc.createRole(forestclient, description)
        # mapping role to user
        kc.addRoleToUser(userid, forestclient)


if __name__ == '__main__':

    # debugging logging
    # LOGGER = logging.getLogger()
    # LOGGER.setLevel(logging.DEBUG)
    # hndlr = logging.StreamHandler()
    # formatter = logging.Formatter(
    # '%(asctime)s - %(name)s - %(levelname)s - %(lineno)d - %(message)s')
    # hndlr.setFormatter(formatter)
    # LOGGER.addHandler(hndlr)

    cli = CLI()
    cli.defineParser()



import logging

import pytest

from . import FOMKeyCloak

LOGGER = logging.getLogger(__name__)
LOGGER.debug("message test")


class Test_FOMKeyCloak:

    def test_getFOMRoles(self, fomKeyCloak_fixture):
        clientRoleExists = fomKeyCloak_fixture.roleExists('234234')
        assert not clientRoleExists

        clientRoleExists = fomKeyCloak_fixture.roleExists('1011')
        assert clientRoleExists

    def test_getRoles(self, fomKeyCloak_fixture):
        roles = fomKeyCloak_fixture.getRoles('99999999')
        # should error but doesn't
        LOGGER.debug(f"roles: {roles}")

    def test_userIDExists(self, fomKeyCloak_fixture):
        userData = fomKeyCloak_fixture.isValidUser('kjnether@idir')
        LOGGER.debug(f"userData: {userData}")
        assert userData is True

    def test_addRoleToUser(self, fomKeyCloak_fixture):
        fomKeyCloak_fixture.addRoleToUser('kjnether@idir', '99999999')

    def test_createRole(self, fomKeyCloak_fixture):
        testfcid = '99999999'
        description = "test dummy role"
        fomKeyCloak_fixture.createRole(testfcid, description)

    def test_getFomClientId(self, fomKeyCloak_fixture):
        client = fomKeyCloak_fixture.getFomClientId()
        LOGGER.debug(f"clientid: {client}")
        assert client is not None

    def test_isValidUser(self, fomKeyCloak_fixture):
        users = fomKeyCloak_fixture.getAllUsers()
        valid = fomKeyCloak_fixture.isValidUser(users[0]['username'])
        LOGGER.debug(f"isvalid: {valid}")


@pytest.fixture
def fomKeyCloak_fixture():
    return FOMKeyCloak.FomKeycloak()
